/* Header for module control, generated by p2c 1.21alpha-07.Dec.93 */
#ifndef CONTROL_H
#define CONTROL_H


#ifdef CONTROL_G
# define vextern
#else
# define vextern extern
#endif
/* DPL 2004-03-21 */
/* Controls which M-Tx features are enabled.
   The interface allows one to interrogate the feature state, to set
   features according to the M-Tx level, and to override features given
   the feature name as a character string.
*/


extern Void mtxLevel PP((Char *level));
extern boolean setFeature PP((Char *which, int val));
/* return TRUE means OK, return FALSE means there is no such feature */
extern Void printFeatures PP((int anyway));
/* anyway TRUE means print all feature settings, FALSE means only those
   that were set via setFeature */

extern boolean multiFile PV();
extern boolean splitShortcut PV();
extern boolean newWordShortcut PV();
extern boolean doChords PV();
extern boolean doUptext PV();
extern boolean doLyrics PV();
extern boolean unbeamVocal PV();
extern boolean uptextOnRests PV();
extern boolean solfaNoteNames PV();
extern boolean pedanticWarnings PV();
extern boolean ignoreErrors PV();
extern boolean hideBlindSlurs PV();
extern boolean instrumentNames PV();
extern boolean beVerbose PV();
extern boolean debugMode PV();
extern boolean insertDuration PV();
extern boolean rearrangeNote PV();
extern boolean countMacro PV();
extern boolean expandMacro PV();
extern boolean checkPitch PV();


#undef vextern

#endif /*CONTROL_H*/

/* End. */
