/* Header for module preamble, generated by p2c 1.21alpha-07.Dec.93 */
#ifndef PREAMBLE_H
#define PREAMBLE_H


/*$X+*/
/* Interpret preamble paragraph, compose PMX preamble */

#ifndef GLOBALS_H
#include "globals.h"
#endif


#ifdef PREAMBLE_G
# define vextern
#else
# define vextern extern
#endif

extern boolean thisCase PV();
extern Void preambleDefaults PV();
extern Void interpretCommands PV();
extern Void doPreamble PV();
extern Void doPMXpreamble PV();
extern Void respace PV();
extern Void restyle PV();
extern Char *startString PP((Char *Result, int voice));
extern Void augmentPreamble PP((int control_para));
extern boolean omitLine PP((int line));
extern Void nonMusic PV();
extern Void setOnly PP((Char *line));
extern boolean isCommand PP((Char *command));


#define known           true


#undef vextern

#endif /*PREAMBLE_H*/

/* End. */
