/* Header for module uptext, generated by p2c 1.21alpha-07.Dec.93 */
#ifndef UPTEXT_H
#define UPTEXT_H


#ifdef UPTEXT_G
# define vextern
#else
# define vextern extern
#endif
/* Insert and translate uptext */


extern Void initUptext PV();
extern Void clearUptext PV();
extern short uptextLineNo PP((int voice));
extern Void setUptextLineNo PP((int voice, int lno));
extern Void addUptext PP((int voice, boolean *no_uptext, Char *pretex));


#undef vextern

#endif /*UPTEXT_H*/

/* End. */
